@@ -2,6 +2,7 @@ from collections import OrderedDict
 
 from vyper import ast as vy_ast
 from vyper.ast.validation import validate_call_args
+from vyper.context.namespace import validate_identifier
 from vyper.context.types.bases import DataLocation, MemberTypeDefinition
 from vyper.context.types.indexable.mapping import MappingDefinition
 from vyper.context.types.utils import get_type_from_annotation
@@ -32,9 +33,17 @@ class StructDefinition(MemberTypeDefinition):
     def is_dynamic_size(self):
         return any(i for i in self.members.values() if i.is_dynamic_size)
 
+    @property
+    def size_in_bytes(self):
+        return sum(i.size_in_bytes for i in self.members.values())
+
     def compare_type(self, other):
         return super().compare_type(other) and self._id == other._id
 
+    @property
+    def canonical_type(self) -> str:
+        return f"({','.join(i.canonical_type for i in self.members.values())})"
+
 
 class StructPrimitive:
 
@@ -42,6 +51,8 @@ class StructPrimitive:
     _as_array = True
 
     def __init__(self, _id, members):
+        for key in members:
+            validate_identifier(key)
         self._id = _id
         self.members = members
 
