@@ -1,6 +1,7 @@
 from typing import Any, List, Optional, Tuple
 
 from vyper import ast as vy_ast
+from vyper.context.types.function import FunctionVisibility, StateMutability
 from vyper.exceptions import (
     EventDeclarationException,
     FunctionDeclarationException,
@@ -14,9 +15,7 @@ from vyper.parser.function_definitions import (
 from vyper.parser.global_context import GlobalContext
 from vyper.parser.lll_node import LLLnode
 from vyper.signatures import sig_utils
-from vyper.signatures.event_signature import EventSignature
 from vyper.signatures.function_signature import FunctionSignature
-from vyper.signatures.interface import check_valid_contract_interface
 from vyper.typing import InterfaceImports
 from vyper.utils import LOADED_LIMITS
 
@@ -46,12 +45,6 @@ def init_func_init_lll():
     return LLLnode.from_list(["seq"] + LIMIT_MEMORY_SET, typ=None)
 
 
-def parse_events(sigs, global_ctx):
-    for event in global_ctx._events:
-        sigs[event.name] = EventSignature.from_declaration(event, global_ctx)
-    return sigs
-
-
 def parse_external_interfaces(external_interfaces, global_ctx):
     for _interfacename in global_ctx._contracts:
         _interface_defs = global_ctx._contracts[_interfacename]
@@ -94,54 +87,83 @@ def parse_external_interfaces(external_interfaces, global_ctx):
     return external_interfaces
 
 
-def parse_other_functions(
-    o,
-    otherfuncs,
-    sigs,
-    external_interfaces,
-    origcode,
-    global_ctx,
-    default_function,
-    is_contract_payable,
-):
-    sub = ["seq", func_init_lll()]
+def parse_other_functions(o, otherfuncs, sigs, external_interfaces, global_ctx, default_function):
+    # check for payable/nonpayable external functions to optimize nonpayable assertions
+    func_types = [i._metadata["type"] for i in global_ctx._defs]
+    mutabilities = [i.mutability for i in func_types if i.visibility == FunctionVisibility.EXTERNAL]
+    has_payable = next((True for i in mutabilities if i == StateMutability.PAYABLE), False)
+    has_nonpayable = next((True for i in mutabilities if i != StateMutability.PAYABLE), False)
+    is_default_payable = (
+        default_function is not None
+        and default_function._metadata["type"].mutability == StateMutability.PAYABLE
+    )
+    # when a contract has a payable default function and at least one nonpayable
+    # external function, we must perform the nonpayable check on every function
+    check_per_function = is_default_payable and has_nonpayable
+
+    # generate LLL for regular functions
+    payable_func_sub = ["seq"]
+    external_func_sub = ["seq"]
+    internal_func_sub = ["seq"]
     add_gas = func_init_lll().gas
 
-    for _def in otherfuncs:
-        sub.append(
-            parse_function(
-                _def,
-                {**{"self": sigs}, **external_interfaces},
-                origcode,
-                global_ctx,
-                is_contract_payable,
-            )
+    for func_node in otherfuncs:
+        func_type = func_node._metadata["type"]
+        func_lll = parse_function(
+            func_node, {**{"self": sigs}, **external_interfaces}, global_ctx, check_per_function
         )
-        sub[-1].total_gas += add_gas
-        add_gas += 30
-        for sig in sig_utils.generate_default_arg_sigs(_def, external_interfaces, global_ctx):
-            sig.gas = sub[-1].total_gas
+        if func_type.visibility == FunctionVisibility.INTERNAL:
+            internal_func_sub.append(func_lll)
+        elif func_type.mutability == StateMutability.PAYABLE:
+            add_gas += 30
+            payable_func_sub.append(func_lll)
+        else:
+            external_func_sub.append(func_lll)
+            add_gas += 30
+        func_lll.total_gas += add_gas
+        for sig in sig_utils.generate_default_arg_sigs(func_node, external_interfaces, global_ctx):
+            sig.gas = func_lll.total_gas
             sigs[sig.sig] = sig
 
-    # Add fallback function
+    # generate LLL for fallback function
     if default_function:
-        default_func = parse_function(
-            default_function[0],
+        fallback_lll = parse_function(
+            default_function,
             {**{"self": sigs}, **external_interfaces},
-            origcode,
             global_ctx,
-            is_contract_payable,
+            # include a nonpayble check here if the contract only has a default function
+            check_per_function or not otherfuncs,
         )
-        fallback = default_func
     else:
-        fallback = LLLnode.from_list(["revert", 0, 0], typ=None, annotation="Default function")
-    sub.append(["seq_unchecked", ["label", "fallback"], fallback])
-    o.append(["return", 0, ["lll", sub, 0]])
-    return o, sub
+        fallback_lll = LLLnode.from_list(["revert", 0, 0], typ=None, annotation="Default function")
+
+    if check_per_function:
+        external_seq = ["seq", payable_func_sub, external_func_sub]
+    else:
+        # payable functions are placed prior to nonpayable functions
+        # and seperated by a nonpayable assertion
+        external_seq = ["seq"]
+        if has_payable:
+            external_seq.append(payable_func_sub)
+        if has_nonpayable:
+            external_seq.extend([["assert", ["iszero", "callvalue"]], external_func_sub])
+
+    # bytecode is organized by: external functions, fallback fn, internal functions
+    # this way we save gas and reduce bytecode by not jumping over internal functions
+    main_seq = [
+        "seq",
+        func_init_lll(),
+        ["with", "_func_sig", ["mload", 0], external_seq],
+        ["seq_unchecked", ["label", "fallback"], fallback_lll],
+        internal_func_sub,
+    ]
+
+    o.append(["return", 0, ["lll", main_seq, 0]])
+    return o, main_seq
 
 
 # Main python parse tree => LLL method
-def parse_tree_to_lll(source_code: str, global_ctx: GlobalContext) -> Tuple[LLLnode, LLLnode]:
+def parse_tree_to_lll(global_ctx: GlobalContext) -> Tuple[LLLnode, LLLnode]:
     _names_def = [_def.name for _def in global_ctx._defs]
     # Checks for duplicate function names
     if len(set(_names_def)) < len(_names_def):
@@ -159,31 +181,16 @@ def parse_tree_to_lll(source_code: str, global_ctx: GlobalContext) -> Tuple[LLLn
     # Initialization function
     initfunc = [_def for _def in global_ctx._defs if is_initializer(_def)]
     # Default function
-    defaultfunc = [_def for _def in global_ctx._defs if is_default_func(_def)]
+    defaultfunc = next((i for i in global_ctx._defs if is_default_func(i)), None)
     # Regular functions
     otherfuncs = [
         _def for _def in global_ctx._defs if not is_initializer(_def) and not is_default_func(_def)
     ]
 
-    # check if any functions in the contract are payable - if not, we do a single
-    # ASSERT CALLVALUE ISZERO at the start of the bytecode rather than at the start
-    # of each function
-    is_contract_payable = next(
-        (
-            True
-            for i in global_ctx._defs
-            if FunctionSignature.from_definition(i, custom_structs=global_ctx._structs).mutability
-            == "payable"
-        ),
-        False,
-    )
-
     sigs: dict = {}
     external_interfaces: dict = {}
     # Create the main statement
     o = ["seq"]
-    if global_ctx._events:
-        sigs = parse_events(sigs, global_ctx)
     if global_ctx._contracts or global_ctx._interfaces:
         external_interfaces = parse_external_interfaces(external_interfaces, global_ctx)
     # If there is an init func...
@@ -191,37 +198,18 @@ def parse_tree_to_lll(source_code: str, global_ctx: GlobalContext) -> Tuple[LLLn
         o.append(init_func_init_lll())
         o.append(
             parse_function(
-                initfunc[0],
-                {**{"self": sigs}, **external_interfaces},
-                source_code,
-                global_ctx,
-                False,
+                initfunc[0], {**{"self": sigs}, **external_interfaces}, global_ctx, False,
             )
         )
 
     # If there are regular functions...
     if otherfuncs or defaultfunc:
         o, runtime = parse_other_functions(
-            o,
-            otherfuncs,
-            sigs,
-            external_interfaces,
-            source_code,
-            global_ctx,
-            defaultfunc,
-            is_contract_payable,
+            o, otherfuncs, sigs, external_interfaces, global_ctx, defaultfunc,
         )
     else:
         runtime = o.copy()
 
-    if not is_contract_payable:
-        # if no functions in the contract are payable, assert that callvalue is
-        # zero at the beginning of the bytecode
-        runtime.insert(1, ["assert", ["iszero", "callvalue"]])
-
-    # Check if interface of contract is correct.
-    check_valid_contract_interface(global_ctx, sigs)
-
     return LLLnode.from_list(o, typ=None), LLLnode.from_list(runtime, typ=None)
 
 
@@ -230,7 +218,7 @@ def parse_to_lll(
 ) -> LLLnode:
     vyper_module = vy_ast.parse_to_ast(source_code)
     global_ctx = GlobalContext.get_global_context(vyper_module, interface_codes=interface_codes)
-    lll_nodes, lll_runtime = parse_tree_to_lll(source_code, global_ctx)
+    lll_nodes, lll_runtime = parse_tree_to_lll(global_ctx)
 
     if runtime_only:
         return lll_runtime
